@@ -31,7 +31,7 @@ optweight <- function(formula, data = NULL, tols = 0, estimand = "ATE", targets
     #treat.name <- t.c[["treat.name"]]
 
     #Get treat type
-    treat.list[[i]] <- get.treat.type(treat.list[[i]])
+    treat.list[[i]] <- assign.treat.type(treat.list[[i]])
     treat.type <- attr(treat.list[[i]], "treat.type")
 
     if (onetime) {
@@ -102,7 +102,7 @@ optweight <- function(formula, data = NULL, tols = 0, estimand = "ATE", targets
   test.w <- if (is_null(sw)) fit_out$w else fit_out$w*sw
   if (any(is.na(test.w))) stop("Some weights are NA, which means something went wrong.", call. = FALSE)
   if (any(sapply(treat.list, function(t) attr(t, "treat.type") == "continuous"))) {if (sd(test.w)/mean(test.w) > 4) warn <- TRUE}
-  else if (any(sapply(treat.list, function(t) any(vapply(unique(t), function(x) sd(test.w[t == x])/mean(test.w[t == x]) > 4, logical(1L)))))) warn <- TRUE
+  else if (any(sapply(treat.list, function(t) any(vapply(unique(t), function(x) coef.of.var(test.w[t == x]) > 4, logical(1L)))))) warn <- TRUE
   if (warn) warning("Some extreme weights were generated. Examine them with summary() and maybe relax the constraints.", call. = FALSE)
   call <- match.call()
 
@@ -160,7 +160,7 @@ print.optweight <- function(x, ...) {
 print.optweightMSM <- function(x, ...) {
   treat.types <- sapply(x[["treat.list"]], function(y) attr(y, "treat.type"))
 
-  cat("A weightitMSM object\n")
+  cat("An optweightMSM object\n")
   cat(paste0(" - number of obs.: ", length(x[["weights"]]), "\n"))
   cat(paste0(" - sampling weights: ", ifelse(all_the_same(x[["s.weights"]]), "none", "present"), "\n"))
   cat(paste0(" - number of time points: ", length(x[["treat.list"]]), "\n"))
