@@ -2,7 +2,7 @@ optweight.svy.fit <- function(covs, tols = 0, targets, s.weights = NULL, norm =
   args <- list(...)
 
   #Process args
-  args[names(args) %nin% names(formals(rosqp::osqpSettings))] <- NULL
+  args[names(args) %nin% names(formals(osqp::osqpSettings))] <- NULL
   if (is_null(args[["max_iter"]])) args[["max_iter"]] <- 2E5L
   if (is_null(args[["eps_abs"]])) args[["eps_abs"]] <- 1E-8
   if (is_null(args[["eps_rel"]])) args[["eps_rel"]] <- 1E-8
@@ -15,7 +15,7 @@ optweight.svy.fit <- function(covs, tols = 0, targets, s.weights = NULL, norm =
       missing.args[arg] <- TRUE
     }
   }
-  if (any(missing.args)) stop(paste(word.list(names(missing.args)[missing.args]), "must be supplied."), call. = FALSE)
+  if (any(missing.args)) stop(paste(word_list(names(missing.args)[missing.args]), "must be supplied."), call. = FALSE)
 
   if (!all(apply(covs, 2, is.numeric))) stop("All covariates must be numeric.", call. = FALSE)
   covs <- as.matrix(covs)
@@ -28,7 +28,7 @@ optweight.svy.fit <- function(covs, tols = 0, targets, s.weights = NULL, norm =
 
   norm.options <- c("l2", "l1", "linf")
   if (length(norm) != 1 || !is.character(norm) || tolower(norm) %nin% norm.options) {
-    stop(paste0("norm must be ", word.list(norm.options, and.or = "or", quotes = TRUE), "."), call. = FALSE)
+    stop(paste0("norm must be ", word_list(norm.options, and.or = "or", quotes = TRUE), "."), call. = FALSE)
   }
   else norm <- tolower(norm)
 
@@ -89,8 +89,8 @@ optweight.svy.fit <- function(covs, tols = 0, targets, s.weights = NULL, norm =
     lower <- c(H1l, F1l, H2l)
     upper <- c(H1u, F1u, H2u)
 
-    out <- rosqp::solve_osqp(P = P, q = q, A = A, l = lower, u = upper,
-                             pars = do.call(rosqp::osqpSettings, args))
+    out <- osqp::solve_osqp(P = P, q = q, A = A, l = lower, u = upper,
+                             pars = do.call(osqp::osqpSettings, args))
 
     #Get dual vars for balance and target constraints
     G2.indices <- if (is_null(G2)) NULL else (NROW(G1)+NROW(E1)+1):(NROW(G1)+NROW(E1)+NROW(G2))
@@ -142,8 +142,8 @@ optweight.svy.fit <- function(covs, tols = 0, targets, s.weights = NULL, norm =
     lowerz = c(lower, H1lz, jl)
     upperz = c(upper, H1uz, ju)
 
-    out <- rosqp::solve_osqp(P = P, q = q, A = Az, l = lowerz, u = upperz,
-                             pars = do.call(rosqp::osqpSettings, args))
+    out <- osqp::solve_osqp(P = P, q = q, A = Az, l = lowerz, u = upperz,
+                             pars = do.call(osqp::osqpSettings, args))
 
     w <- out$x[1:N]
 
@@ -207,8 +207,8 @@ optweight.svy.fit <- function(covs, tols = 0, targets, s.weights = NULL, norm =
     lowerz = c(lower, H1lz, jl)
     upperz = c(upper, H1uz, ju)
 
-    out <- rosqp::solve_osqp(P = P, q = q, A = Az, l = lowerz, u = upperz,
-                             pars = do.call(rosqp::osqpSettings, args))
+    out <- osqp::solve_osqp(P = P, q = q, A = Az, l = lowerz, u = upperz,
+                             pars = do.call(osqp::osqpSettings, args))
 
     w <- out$x[1:N]
 
